/*
 *  Copyright (C) 2004 Max-Planck-Society
 *  \author Martin Reinecke
 */

#ifndef PLANCK_FFTPACK_SUPPORT_H
#define PLANCK_FFTPACK_SUPPORT_H

#include "ls_fft.h"
#include "arr.h"
#include "xcomplex.h"

class cfft
  {
  private:
    int n;
    complex_plan plan;

  public:
    cfft () : n(-1), plan(0) {}
    cfft (int size_)
      : n(size_), plan(make_complex_plan(size_)) {}
    ~cfft ()
      { if (plan!=0) kill_complex_plan (plan); }
    void Set (int size_)
      {
      if (plan!=0) kill_complex_plan (plan);
      n=size_;
      plan=make_complex_plan(size_);
      }

    int size() const
      { return n; }

    void forward (double *data)
      { complex_plan_forward(plan,data); }
    void backward (double *data)
      { complex_plan_backward(plan,data); }
    void forward (arr<xcomplex<double> >&data)
      { forward(&(data[0].re)); }
    void backward (arr<xcomplex<double> >&data)
      { backward(&(data[0].re)); }
  };

class rfft
  {
  private:
    int n;
    real_plan plan;

  public:
    rfft () : n(-1), plan(0) {}
    rfft (int size_)
      : n(size_), plan(make_real_plan(size_)) {}
    ~rfft ()
      { if (plan!=0) kill_real_plan (plan); }
    void Set (int size_)
      {
      if (plan!=0) kill_real_plan (plan);
      n=size_;
      plan=make_real_plan(size_);
      }

    int size() const
      { return n; }

    void forward_fftpack (double *data)
      { real_plan_forward_fftpack(plan,data); }
    void backward_fftpack (double *data)
      { real_plan_backward_fftpack(plan,data); }
    void forward_fftpack (arr<double> &data)
      { forward_fftpack(&(data[0])); }
    void backward_fftpack (arr<double> &data)
      { backward_fftpack(&(data[0])); }
    void forward_c (arr<xcomplex<double> >&data)
      { real_plan_forward_c(plan,&(data[0].re)); }
    void backward_c (arr<xcomplex<double> >&data)
      { real_plan_backward_c(plan,&(data[0].re)); }
  };

#endif
